@@ -5,16 +5,15 @@
 module Data.Conduit.Process.Typed
   ( -- * Conduit specific stuff
     createSink
+  , createSinkClose
   , createSource
-    -- * Generalized functions
-  , withProcess -- FIXME import from rio instead
-  , withProcess_
+    -- * Running a process with logging
   , withLoggedProcess_
     -- * Reexports
   , module System.Process.Typed
   ) where
 
-import System.Process.Typed hiding (withProcess, withProcess_)
+import System.Process.Typed
 import qualified System.Process.Typed as P
 import Data.Conduit (ConduitM, (.|), runConduit)
 import qualified Data.Conduit.Binary as CB
@@ -25,9 +24,13 @@ import qualified Data.ByteString.Lazy as BL
 import Data.IORef (IORef, newIORef, readIORef, modifyIORef)
 import Control.Exception (throwIO, catch)
 import Control.Concurrent.Async (concurrently)
-import System.IO (hSetBuffering, BufferMode (NoBuffering))
+import System.IO (hSetBuffering, BufferMode (NoBuffering), hClose)
 
--- | Provide input to a process by writing to a conduit.
+-- | Provide input to a process by writing to a conduit. The sink provided here
+-- will leave the pipe to the child open after the stream ends. This allows the
+-- sink to be used multiple times, but may result in surprising behavior. You
+-- may prefer 'createSinkClose', see
+-- <https://github.com/snoyberg/conduit/issues/434>.
 --
 -- @since 1.2.1
 createSink :: MonadIO m => StreamSpec 'STInput (ConduitM S.ByteString o m ())
@@ -35,6 +38,15 @@ createSink =
   (\h -> liftIO (hSetBuffering h NoBuffering) >> CB.sinkHandle h)
   `fmap` createPipe
 
+-- | Like 'createSink', but closes the pipe to the child process as soon as it
+-- runs out of data.
+--
+-- @since 1.3.5
+createSinkClose :: MonadIO m => StreamSpec 'STInput (ConduitM S.ByteString o m ())
+createSinkClose =
+  (\h -> liftIO (hSetBuffering h NoBuffering) >> CB.sinkHandle h >> liftIO (hClose h))
+  `fmap` createPipe
+
 -- | Read output from a process by read from a conduit.
 --
 -- @since 1.2.1
@@ -58,26 +70,6 @@ createSourceLogged ref =
     )
     `fmap` createPipe
 
--- | Same as 'P.withProcess', but generalized to 'MonadUnliftIO'.
---
--- @since 1.2.1
-withProcess
-  :: MonadUnliftIO m
-  => ProcessConfig stdin stdout stderr
-  -> (Process stdin stdout stderr -> m a)
-  -> m a
-withProcess pc f = withRunInIO $ \run -> P.withProcess pc (run . f)
-
--- | Same as 'P.withProcess_', but generalized to 'MonadUnliftIO'.
---
--- @since 1.2.1
-withProcess_
-  :: MonadUnliftIO m
-  => ProcessConfig stdin stdout stderr
-  -> (Process stdin stdout stderr -> m a)
-  -> m a
-withProcess_ pc f = withRunInIO $ \run -> P.withProcess_ pc (run . f)
-
 -- | Run a process, throwing an exception on a failure exit code. This
 -- will store all output from stdout and stderr in memory for better
 -- error messages. Note that this will require unbounded memory usage,
@@ -97,7 +89,10 @@ withLoggedProcess_ pc inner = withUnliftIO $ \u -> do
   stderrBuffer <- newIORef id
   let pc' = setStdout (createSourceLogged stdoutBuffer)
           $ setStderr (createSourceLogged stderrBuffer) pc
-  P.withProcess pc' $ \p -> do
+  -- withProcessWait vs Term doesn't actually matter here, since we
+  -- call checkExitCode inside regardless. But still, Wait is the
+  -- safer function to use in general.
+  P.withProcessWait pc' $ \p -> do
     a <- unliftIO u $ inner p
     let drain src = unliftIO u (runConduit (src .| CL.sinkNull))
     ((), ()) <- drain (getStdout p) `concurrently`
